#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

#include "common.h"
#include "util.h"
#include "draw.h"


#define TMP_FILE	"out.tmp"

#define COMMAND \
  "gs -sDEVICE=ppmraw -sOutputFile=- -g480x480 -r72 -q - |" \
  "ppmtopgm | pamditherbw | pamtopnm | sed 1,2d >" TMP_FILE


static FILE *file;


void begin_draw(int for_print)
{
    if (for_print)
	file = stdout;
    else
	file = popen(COMMAND, "w");
    fprintf(file, "%%!PS\n");
    if (for_print)
	fprintf(file, "200 200 translate 72 %d div dup scale\n", DPI);
    fprintf(file, "%d %d translate\n", XSIZE/2, YSIZE/2);
    fprintf(file, "6 setlinewidth\n");
    fprintf(file,
      "/hcenter { dup stringwidth pop 2 div neg 0 rmoveto show newpath } "
      "def\n");
}


void draw_circle(int r)
{
    fprintf(file, "0 0 %d 0 360 arc stroke\n",r);
}


void draw_spoke(int r0, int r1, int angle)
{
    fprintf(file, "-%d rotate\n", angle);
    fprintf(file, "0 %d moveto 0 %d rlineto stroke\n", r0, r1);
    fprintf(file, "%d rotate\n", angle);
}


void draw_text(int r, int angle, const char *text, int size)
{
    fprintf(file, "/Helvetica-Bold %d selectfont\n", size);
    fprintf(file, "-%d rotate\n", angle);
    fprintf(file, "0 %d moveto\n", r);
    fprintf(file, "%d rotate\n", angle);
    fprintf(file, "0 -%d rmoveto\n", (int) (size*0.35));
    fprintf(file, "(%s) hcenter\n", text);
}


static uint8_t *read_file(const char *name)
{
    int fd;
    uint8_t *buf;
    struct stat st;
    size_t done;
    ssize_t got;

    fd = open(name, O_RDONLY);
    if (fd < 0) {
	perror(TMP_FILE);
	exit(1);
    }
    if (fstat(fd, &st) < 0) {
	perror(TMP_FILE);
	exit(1);
    }
    buf = alloc_type_n(uint8_t, st.st_size+1);
    buf[st.st_size] = 0;
    for (done = 0; done != st.st_size; done += got) {
	got = read(fd, buf+done, st.st_size-done);
	if (got < 0) {
	    perror("read");
	    exit(1);
	}
	if (!got) {
	    fprintf(stderr, "unexpected EOF\n");
	    exit(1);
	}
    }
    close(fd);
    return buf;
}


const uint8_t *end_draw(void)
{
    static uint8_t reverse[256];
    uint8_t *data;
    int i, j;

    fprintf(file, "showpage\n");
    if (file == stdout)
	return NULL;
    for (i = 0; i != 256; i++)
	for (j = 0; j != 8; j++)
	    if ((i >> j) & 1)
		reverse[i] |= 1 << (7-j);
    if (pclose(file)) {
	perror("pclose");
	exit(1);
    }
    data = read_file(TMP_FILE);
    for (i = 0; i != XSIZE*YSIZE/8; i++)
	data[i] = reverse[data[i]];
return data;
    if (unlink(TMP_FILE) < 0) {
	perror("unlink " TMP_FILE);
	exit(1);
    }
    return data;
}
