/*
 * error.c - Error reporting for programming languages
 *
 * Written 2006-2007 by Werner Almesberger
 * Copyright 2006-2007 Werner Almesberger
 */


#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>

#include "error.h"


int lineno = 0;


/* ----- Errors ------------------------------------------------------------ */


static void __attribute__((noreturn)) verrorf(const char *fmt,va_list ap)
{
    extern char *yytext;

    fflush(stdout);
    fprintf(stderr,"%d: ",lineno);
    vfprintf(stderr,fmt,ap);
    fprintf(stderr," (near \"%s\")",yytext);
    putc('\n',stderr);
    exit(1);
}


void __attribute__((noreturn)) yyerrorf(const char *fmt,...)
{
    va_list ap;

    va_start(ap,fmt);
    verrorf(fmt,ap);
    va_end(ap);
}


void __attribute__((noreturn)) yyerror(const char *s)
{
    yyerrorf("%s",s);
}


#if 0
/* ----- Warnings ---------------------------------------------------------- */


void  vlwarnf(const struct loc *loc,const char *fmt,va_list ap)
{
    fflush(stdout);
    fprintf(stderr,"%s:%d: warning: ",loc->file,loc->line);
    vfprintf(stderr,fmt,ap);
    putc('\n',stderr);
}


void lwarnf(const struct loc *loc,const char *fmt,...)
{
    va_list(ap);

    va_start(ap,fmt);
    vlwarnf(loc,fmt,ap);
    va_end(ap);
}


void yywarnf(const char *fmt,...)
{
    va_list ap;

    va_start(ap,fmt);
    vlwarnf(&current_loc,fmt,ap);
    va_end(ap);
}


void yywarn(const char *s)
{
    lwarnf(&current_loc,"%s",s);
}


void error_init(void)
{
    file_names = make_jrb();
}


void error_cleanup(void)
{
    JRB entry;

    jrb_traverse(entry,file_names)
	free(jval_v(jrb_val(entry)));
    jrb_free_tree(file_names);
}
#endif
