#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "common.h"
#include "cpp.h"
#include "layout.h"
#include "map.h"


int debug = 0;


int yyparse(void);


static void usage(const char *name)
{
    fprintf(stderr,
"usage: %s [-d ...] [cpp_option ...] [setup_file]\n"
"       %s -p [-d ...] [cpp_option ...] [setup_file]\n"
"       %s -r [-d ...] [cpp_option ...] [setup_file]\n"
"       %s -w [-d ...] [cpp_option ...] [setup_file]\n"
"\n"
"  -d ...      enable debugging functions (repeat for more output)\n"
"  -p          print layout in Postscript to stdout\n"
"  -r          read layout from .iwl files, then proceed normally\n"
"  -w          write layout to .iwl files\n"
"  cpp_option  -Idir, -Dname[=value], or -Uname\n", name, name, name, name);
    exit(1);
}


int main(int argc, char **argv)
{
    int print_layout = 0;
    int read_layout = 0;
    int write_layout = 0;
    int c;

    gtk_init(&argc, &argv);
    layout_init();

    while ((c = getopt(argc, argv, "dprwD:I:U:")) != EOF) {
	char opt[] = "-?";

	switch (c) {
	    case 'd':
		debug++;
		break;
	    case 'p':
		print_layout = 1;
		break;
	    case 'r':
		read_layout = 1;
		break;
	    case 'w':
		write_layout = 1;
		break;
	    case 'D':
	    case 'I':
	    case 'U':
		opt[1] = c;
		add_cpp_arg(opt);
		add_cpp_arg(optarg);
		break;
	}
    }

    if (print_layout+read_layout+write_layout > 1)
	usage(*argv);

    switch (argc-optind) {
	case 0:
	    run_cpp_on_file(NULL);
	    break;
	case 1:
	    run_cpp_on_file(argv[optind]);
	    break;
	default:
	    usage(*argv);
    }
    yyparse();
    reap_cpp();

    layout_display("base");
    map_select("base");
    while (1)
	gtk_main();

    return 0;
}
