%{
/*
 * lang.l - Input Wheel language
 *
 * Written 2007 by Werner Almesberger
 * Copyright 2007 Werner Almesberger
 */


#include <stdlib.h>
#include <string.h>

#include "util.h"
#include "error.h"
#include "backslash.h"

#include "y.tab.h"


void yyerror(const char *s);


%}

%%

accept				return TOK_ACCEPT;
center				return TOK_CENTER;
exit				return TOK_EXIT;
field				return TOK_FIELD;
layout				return TOK_LAYOUT;
map				return TOK_MAP;
neutral				return TOK_NEUTRAL;
polar				return TOK_POLAR;
radius				return TOK_RADIUS;
reset				return TOK_RESET;
segments			return TOK_SEGMENTS;
send				return TOK_SEND;
size				return TOK_SIZE;
spokes				return TOK_SPOKES;
tr				return TOK_TR;
wheel				return TOK_WHEEL;
zero				return TOK_ZERO;

"->"				return TOK_ARROW;

[0-9]+				{ char *end;
				  yylval.num = strtoul(yytext, &end, 10);
				  if (*end)
				    yyerror("invalid digit in number");
				  return NUM; }

[a-zA-Z_][a-zA-Z0-9]*		{ yylval.str = stralloc(yytext);
				  return ID; }

\"(\\[^\n\t]|[^\\"\n\t])*\"     { char *s;
				  *strrchr(yytext,'"') = 0;
				  yylval.str = stralloc(yytext+1);
				  backslash((char *) yylval.str);
                                  return STRING; }

[\t ]*
\n				lineno++;

^#\ [0-9]+\ \"[^"]*\"(\ [0-9]+)*\n {
				  char *start = strchr(yytext,'"')+1;
				  *strchr(start,'"') = 0;
				  // set_file(start);
				  lineno = strtol(yytext+2,NULL,0); }

.                               return *yytext;
