#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <gtk/gtk.h>

#include "common.h"
#include "util.h"
#include "pattern.h"
#include "layout.h"


static struct layout {
    const char *name;
    GdkPixmap *pixmap;
    struct layout *next;
} *layouts = NULL;


static GtkWidget *window;
static GtkWidget *image;
static int xcenter, ycenter;


#if 0
gdk_draw_pixmap(widget->window,
                  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                  pixmap,
                  event->area.x, event->area.y,
                  event->area.x, event->area.y,
                  event->area.width, event->area.height);
#endif


/*
 * Stolen from http://www.gtk.org/~otaylor/gtk/tutorial/drawing_tut-1.html
 *
 * @@@FIXME:
 * Note that we actually clobber the last layout that's been loaded. Not nice.
 */

void layout_debug_mark(int x, int y, int active)
{
    GtkWidget *widget = image;
    GdkRectangle update_rect;

    x += xcenter;
    y = ycenter-y;
    update_rect.x = x - 5;
    update_rect.y = y - 5;
    update_rect.width = 10;
    update_rect.height = 10;
    gdk_draw_rectangle(layouts->pixmap,
      active ? widget->style->black_gc : widget->style->mid_gc[0],
      TRUE,
      update_rect.x, update_rect.y, update_rect.width, update_rect.height);
    gtk_widget_draw(widget, &update_rect);
    gdk_draw_pixmap(window->window,
      widget->style->fg_gc[GTK_WIDGET_STATE(widget)], layouts->pixmap,
      x-5,y-5,x-5,y-5,10,10);
}


static gboolean button_press_event(GtkWidget *widget, GdkEventButton *event)
{
    if (event->button == 1)
	pattern_press(event->x-xcenter, ycenter-event->y);
    return TRUE;
}


static gboolean button_release_event(GtkWidget *widget, GdkEventButton *event)
{
    if (event->button == 1)
	pattern_release(event->x-xcenter, ycenter-event->y);
    return TRUE;
}


static gboolean motion_notify_event(GtkWidget *widget, GdkEventMotion *event)
{
    assert(!event->is_hint);

    if (event->state & GDK_BUTTON1_MASK)
	pattern_track(event->x-xcenter, ycenter-event->y);
    return TRUE;
}


void layout_init(void)
{
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_name(window, "Input Wheel");

    image = gtk_image_new();
    gtk_widget_set_size_request(GTK_WIDGET(image), XSIZE, YSIZE);
    //gtk_container_add(GTK_CONTAINER(window), image);

    gtk_container_add(GTK_CONTAINER(window), image);

    gtk_widget_show(image);

    g_signal_connect(G_OBJECT(window), "motion_notify_event",
      G_CALLBACK(motion_notify_event), NULL);
    g_signal_connect(G_OBJECT(window), "button_press_event",
      G_CALLBACK(button_press_event), NULL);
    g_signal_connect(G_OBJECT(window), "button_release_event",
      G_CALLBACK(button_release_event), NULL);

    gtk_widget_set_events(window,
      GDK_LEAVE_NOTIFY_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
      GDK_POINTER_MOTION_MASK);

    gtk_widget_show(window);

    xcenter = XSIZE/2;
    ycenter = YSIZE/2+YERROR;
}


void layout_store(const char *name, const uint8_t *xbm)
{
    struct layout *layout;
    GtkStyle *style;

    layout = alloc_type(struct layout);
    layout->name = name;
    style = gtk_widget_get_style(window);
    layout->pixmap = gdk_pixmap_create_from_data(window->window, (void *) xbm,
       XSIZE, YSIZE, -1, &style->black, &style->white);
    layout->next = layouts;
    layouts = layout;
}


static void show_layout(GdkPixmap *pixmap)
{
    gtk_image_set_from_pixmap(GTK_IMAGE(image), pixmap, NULL);
}


void layout_display(const char *name)
{
    struct layout *layout;

    for (layout = layouts; layout; layout = layout->next)
	if (!strcmp(layout->name, name)) {
	    show_layout(layout->pixmap);
	    return;
	}
    fprintf(stderr, "layout \"%s\" not found\n", name);
    exit(1);
}
