#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "common.h"
#include "util.h"
#include "wheel.h"
#include "action.h"
#include "layout.h"
#include "pattern.h"


int neutral_center = 0;


struct pattern {
    const struct action *action;
    struct pattern *next[MAX_ZONES];
} base, *curr;

static int last_zone;


void pattern_start(void)
{
    curr = &base;
}


static void provide(struct pattern **p)
{
    if (!*p) {
	*p = alloc_type(struct pattern);
	memset(*p, 0, sizeof(struct pattern));
    }
}


void pattern_enter(int zone)
{
    provide(curr->next+zone);
    curr = curr->next[zone];
}


void pattern_clear(void)
{
    /* @@@FIXME: the mother of all memory leaks ... */
    memset(&base, 0, sizeof(base));
}


const struct pattern *pattern_get(void)
{
    struct pattern *tmp;

    tmp = alloc_type(struct pattern);
    *tmp = base;
    return tmp;
}


static void recurse(struct pattern *to, const struct pattern *from)
{
    int i;

    if (from->action)
	to->action = from->action;
    for (i = 0; i != MAX_ZONES; i++)
	if (from->next[i]) {
	     provide(to->next+i);
	     recurse(to->next[i], from->next[i]);
	}
}


void pattern_merge(const struct pattern *p)
{
    recurse(&base, p);
}


void pattern_action(const struct action *action)
{
    curr->action = action;
}


static gint timeout = 0;


static gboolean deferred_action(gpointer data)
{
    const struct pattern *c = data;
    const struct action *act = c ? c->action : NULL;

    timeout = 0;
    if (act)
	do_actions(act);
    gtk_main_quit();
    return FALSE;
}


static void cancel_deferred(void)
{
    if (timeout) {
	g_source_remove(timeout);
	timeout = 0;
    }
}



static void defer(void)
{
    timeout = g_timeout_add(DEBOUNCE_MS, deferred_action, curr);
}


void pattern_press(int x, int y)
{
    int zone;

    if (debug > 1)
	fprintf(stderr, "PRESS %d %d %d\n", x, y, wheel_zone(x, y));
    if (timeout) {
	cancel_deferred();
	pattern_track(x, y);
    }
    else {
	zone = last_zone = wheel_zone(x, y);
	curr = zone == -1 ? NULL : base.next[zone];
    }
    if (debug)
	layout_debug_mark(x, y, !!curr);
}


/* @@@FIXME: if neutral_center is set, we can also enter zones from -1, not
   only from 0. */


void pattern_track(int x, int y)
{
    int zone;

    if (debug > 1)
	fprintf(stderr, "TRACK %d %d %d\n", x, y, wheel_zone(x, y));
    zone = wheel_zone(x, y);
    if (zone == -1) {
	curr = NULL;
	return;
    }
    if (neutral_center) {
	if (curr && !zone) {
	    defer();
	    curr = NULL;
	    return;
	}
	if (!curr && zone)
	    curr = base.next[zone];
    }
    if (!curr)
	    return;
    if (zone != last_zone) {
	last_zone = zone;
	curr = curr->next[zone];
    }
    if (debug)
	layout_debug_mark(x, y, !!curr);
}


void pattern_release(int x, int y)
{
    pattern_track(x, y);
    if (debug > 1)
	fprintf(stderr, "RELEASE %d %d %d\n", x, y, wheel_zone(x, y));
    defer();
}
