#include <math.h>

#include "draw.h"
#include "wheel.h"


#define MAX_FIELDS	20	/* @@@FIXME: overflow check */


static int spokes, angle0, r_c = 0, r_s;


static struct field {
    int size;
    int radius;
    int angle;
    const char *label;
} *field, f_c[MAX_FIELDS], f_s[MAX_FIELDS];


void wheel_init(int n_spokes, int start_angle)
{
    spokes = n_spokes;
    angle0 = start_angle;
}


void wheel_center(int radius)
{
    r_c = radius;
    field = f_c;
}


void wheel_segment(int radius)
{
    r_s = radius;
    field = f_s;
}


void wheel_field(int size, int radius, int angle)
{
    field->size = size;
    field->radius = radius;
    field->angle = angle;
    field++;
}


void wheel_draw(void)
{
    int i;

    draw_circle(r_c);
    draw_circle(r_c+r_s);
    for (i = 0; i != spokes; i++)
	draw_spoke(r_c, r_s, angle0+360*i/spokes);
}


void wheel_label(int zone, int nth, const char *label)
{
    if (zone)
	draw_text(r_c+f_s[nth].radius,
	  angle0+360*(zone-1)/spokes+f_s[nth].angle,
	  label, f_s[nth].size);
    else
	draw_text(f_c[nth].radius, f_c[nth].angle, label, f_c[nth].size);
}


#define PI 3.1415926


int wheel_zone(int x, int y)
{
    int r, angle;

    r = hypot(x, y);
    if (r <= r_c)
	return 0;
    if (r > r_c+r_s)
	return -1;
    angle = ((int) ((atan2(x, y)+4.0*PI)/PI*180.0)-angle0) % 360;
    return angle/(360/spokes)+1;
}
